\ProvidesClass{homework}
\NeedsTeXFormat{LaTeX2e}
\DeclareOption*{\PassOptionsToClass{\CurrentOption}{article}}
\ProcessOptions\relax
\LoadClass{article}

%%%%%%%%%%%%%%%
% MacroMaking %
%%%%%%%%%%%%%%%

\RequirePackage{xparse} %more robust command/environment making
\RequirePackage{luacode} %for easier use of embedded lua

%%%%%%%%%%%%%%%%%%%%%%
% Document Structure %
%%%%%%%%%%%%%%%%%%%%%%

\RequirePackage{geometry}
    \geometry{%
        top=0.5in,
        bottom=0.5in,
        left=0.75in,
        textwidth=7in,
        paperwidth=8.5in,
        paperheight=11in,
        hmarginratio={1:1},
        heightrounded
    }

%%%%%%%%%
% Fonts %
%%%%%%%%%

\RequirePackage{mathtools}
\RequirePackage{microtype}
\usepackage[mathrm=sym,
            mathbf=sym,
            warnings-off={mathtools-colon,
                          mathtools-overbracket
                         }
            ]{unicode-math}
\setmainfont[Scale=1]{XITS}
\setmathfont[Scale=MatchLowercase,
            ]{XITS Math}
\setsansfont[Scale=MatchLowercase]{Roboto}
%%%%%%%%%%%%
% Graphics %
%%%%%%%%%%%%

\RequirePackage{tikz}
    \usetikzlibrary{
        calc,
        positioning,
        fit,
        intersections
    }
    \pgfdeclarelayer{bg}    % declare background layer
    \pgfsetlayers{bg,main}  % set the order of the layers (main is the standard layer)

\RequirePackage{pgfplots}
    \pgfplotsset{compat=1.18}
\RequirePackage{asymptote}
    \def\asydir{pics}

%NewEnvironmentsCommands

\newcounter{@prob}
\setcounter{@prob}{1}
\DeclareDocumentCommand{\problem}{m O{\the@prob}}{%
    \vskip0.25cm
    \noindent {\bfseries Problem #2.} (#1 points)
    \stepcounter{@prob}
}

%%%%%%%%
% Misc %
%%%%%%%%

\renewcommand*\env@matrix[1][*\c@MaxMatrixCols c]{%
  \hskip -\arraycolsep
  \let\@ifnextchar\new@ifnextchar
  \array{#1}}

%%%%%%%%%%
% Header %
%%%%%%%%%%

\def\@class{empty}
\newcommand{\setclass}[1]{\def\@class{#1}}
\newcommand{\show@class}{\sffamily\bfseries\@class}
\def\@assignment{}
\newcommand{\setassignment}[1]{\def\@assignment{#1}}
\newcommand{\show@assignment}{\sffamily\bfseries\@assignment}
\def\@duedate{}
\newcommand{\setduedate}[1]{\def\@duedate{#1}}
\newcommand{\show@duedate}{\sffamily\bfseries\@duedate}
\def\@suggested{}
\newcommand{\setsuggested}[1]{\def\@suggested{#1}}
\newcommand{\show@suggested}{\@suggested}
\def\@docname{"\jobname.tex"} %Overleaf has strange behavior here
\newcommand{\makebanner}{%
\noindent%
\luaexec{
    doc_name   = \@docname
    doc        = io.open(doc_name,"r")
    doc_cont   = doc:read("*a")
    point_vals = {}
    for word in string.gmatch(doc_cont,"\\problem{\%d+}") do
        table.insert(point_vals,string.match(word,"\%d+"))
    end
    doc:close()
}%
\begin{center}
  \begin{tikzpicture}
    \node[anchor=north,
          rectangle,
          inner sep=0pt,
          outer sep=0pt,
          text width=\textwidth]
            (box) at (current bounding box.north) {};
    \node[anchor=north west,
        rectangle] (class) at (box.north west) {\huge\show@class};
    \node[anchor=north east,rectangle,rounded corners=1pt]
        (scores) at (box.north east) {
            \begin{tikzpicture}[scale=0.8]
            \luaexec{
            tex.print("\\draw [densely dashed,rounded corners] (0,3) grid (",\#point_vals,"+1,0);")
            }
            \node[anchor=east] at (0,2.5) { \bfseries Problems:};
            \node[anchor=east] at (0,1.5) {Points:};
            \node[anchor=east] at (0,0.5) {Scores:};
            \luaexec{
                for i=1, \#point_vals do
                tex.print("\\node[anchor=center] at (",i,"-0.5,2.5) {\\bf",i,"};")
                tex.print("\\node[anchor=center] at (",i,"-0.5,1.5) {",point_vals[i],"};")
                end
                local sum = 0
                for i=1, \#point_vals do
                    sum = sum + point_vals[i]
                end
                sum=math.floor(sum)
                tex.print(
                    "\\node[anchor=center] at (",\#point_vals,"+0.5,2.5) {\\textbf{Tot}};"
                )
                tex.print(
                    "\\node[anchor=center] at (",\#point_vals,"+0.5,1.5) {",sum,"};"
                )
            }
            \end{tikzpicture}
        };
    \node[anchor=north west,
        rectangle,
        align=left] (assignment) at (class.south west) {
        \show@assignment \\
        \show@duedate
        };
    \draw let \p1 = ($(scores.north west)-(class.north west)$),
              \n1 = {0.95*veclen(\x1,\y1)}
          in node[anchor=north west,
                  rectangle,
                  align=left,
                  text width=\n1] (suggested) at (assignment.south west) {
                  \show@suggested
                  };
    \node[anchor=north west,
        rectangle,
        align=left] (namecm) at (suggested.south west) {
             \bfseries Name/CM:};
    \begin{pgfonlayer}{bg}
        \node[overlay,rectangle,
            draw,
            fit=(class)(suggested)(assignment)(scores)(namecm),
            rounded corners=1pt,
            inner sep=0pt,
            outer sep=0pt,
            very thick,
            darkgray] (banner) {};
    \end{pgfonlayer}
    \begin{scope}
    \clip (banner.north west) rectangle (banner.south east);
    \path[name path=box] (banner.south west) rectangle (banner.north east);
    \path [name path=divider] (scores.north west) --+ (-90:10);
    \draw [darkgray,
           very thick,
           name intersections={of=box and divider}]
           (scores.north west) -- (intersection-2);
    \end{scope}
    \end{tikzpicture}
\end{center} \vskip0.15cm
}

%%%%%%%%%%%%%%%%%%%%%%
% boxed environments %
%%%%%%%%%%%%%%%%%%%%%%

\RequirePackage[most]{tcolorbox}

\NewDocumentEnvironment{solution}{}{
    \begin{tcolorbox}[enhanced,
                      breakable,
                      colback=Maroon!5,
                      colframe=brown,
                      coltitle=white,
                      title={ \textbf{Solution}}]
    }{
    \end{tcolorbox}
    }