import texsettings;
import smoothcontour3;
import graph3; import grid3; import three; import tube;
import animate; import labelpath3; import contour; import
palette;


size(8cm,0);
currentprojection=orthographic(1,4,3);
currentlight=light(lightgray,
                  specularfactor=1,
                   (-1,3,3),(3,-1,3));


real f(real x, real y){
    return 2-cos(x)*sin(x*y);
}

real r(real theta){
    return 2;
}

triple s(pair uv){
    real theta=uv.x; real w=uv.y;
    return (w*r(theta)*cos(theta),
            w*r(theta)*sin(theta),
            f(w*r(theta)*cos(theta),
                w*r(theta)*sin(theta)));
}

real x(real theta){
    return r(theta)*cos(theta);
}
real y(real theta){
    return r(theta)*sin(theta);
}
real z(real theta){
    return 0;
}

splinetype[] Notaknot={notaknot,notaknot,notaknot};

surface surf=surface(s,(0,0),(pi/2,1),nu=20,nv=20,Notaknot,Notaknot);
path3 p=graph(x,y,z,0,pi/2,operator..);

draw(surf,rosewhite+opacity(0.25),meshpen=rose+0.25);
draw(p,rose+dashed);

for(int i=5;i<15;++i){
for(int k=5;k<15;++k){

real angle=0+i*pi/40;
real rad=0.05+0.05*k;

triple top(pair uv){
    real theta=uv.x; real w=uv.y;
    return (s((theta,w)).x,
            s((theta,w)).y,
            s((angle,rad)).z);
}
triple fedge(pair uv){
    real theta=uv.x; real h=uv.y;
    return (s((theta,rad)).x,
            s((theta,rad)).y,
            h*s((angle,rad)).z);
}
triple bedge(pair uv){
    real theta=uv.x; real h=uv.y;
    return (s((theta,rad-0.05)).x,
            s((theta,rad-0.05)).y,
            h*s((angle,rad)).z);
}
triple lside(pair uv){
    real w=uv.x; real h=uv.y;
    return (s((angle,rad-w*0.05)).x,
            s((angle,rad-w*0.05)).y,
            h*s((angle,rad)).z);
}
triple rside(pair uv){
    real w=uv.x; real h=uv.y;
    return (s((angle+pi/40,rad-w*0.05)).x,
            s((angle+pi/40,rad-w*0.05)).y,
            h*s((angle,rad)).z);
}

draw(surface(
    top,(angle,rad-0.05),(angle+pi/40,rad),nu=1,nv=1,Spline),
    rose,meshpen=black+0.5);
draw(surface(
    fedge,(angle,0),(angle+pi/40,1),nu=1,nv=1,Spline),
    rosewhite,meshpen=black+0.5);
draw(surface(
    bedge,(angle,0),(angle+pi/40,1),nu=1,nv=1,Spline),
    rosewhite,meshpen=black+0.5);
draw(surface(
    lside,(0,0),(1,1),nu=1,nv=1,Spline),
    rosewhite,meshpen=black+0.5);
draw(surface(
    rside,(0,0),(1,1),nu=1,nv=1,Spline),
    rosewhite,meshpen=black+0.5);
}
}

for(int i=5;i<16;++i){
    real xt(real theta){
        return (i*0.1)*cos(theta);
    }
    real yt(real theta){
        return (i*0.1)*sin(theta);
    }
    real zt(real theta){
        return 0 ;
    }
    real xr(real t){
        return t*cos(i*pi/40);
    }
    real yr(real t){
        return t*sin(i*pi/40);
    }
    real zr(real t){
        return 0 ;
    }
    path3 pt=graph(xt,yt,zt,0,pi/2,operator..);
    path3 pr=graph(xr,yr,zr,0,2,operator..);
    draw(pt,rose+dashed);
    draw(pr,rose+dashed);
}

label(rotate(45,Z)*planeproject(Z)*
    "$\theta_i$",
    (2.1cos(pi/4),2.1sin(pi/4),0),
    rose,align=E,Embedded);

label(rotate(50,Z)*planeproject(Z)*
    "$\theta_{i+1}$",
    (2.1cos(5pi/18),2.1sin(5pi/18),0),
    rose,align=E,Embedded);
    
label(rotate(22.5,Z)*planeproject(Z)*
    "$\alpha$",
    (2.1cos(5pi/40),2.1sin(5pi/40),0),
    rose,align=E,Embedded);
    
label(rotate(67.5,Z)*planeproject(Z)*
    "$\beta$",
    (2.1cos(15pi/40),2.1sin(15pi/40),0),
    rose,align=E,Embedded);

label(planeproject(Z)*
    "$\iota$",
    (0,0.5,0),
    rose,align=W,Embedded);

label(planeproject(Z)*
    "$o$",
    (0,1.5,0),
    rose,align=W,Embedded);
    
label(planeproject(Z)*
    "$r_{k-1}$",
    (0,1,0),
    rose,align=W,Embedded);
    
label(planeproject(Z)*
    "$r_k$",
    (0,1.2,0),
    rose,align=W,Embedded);


xaxis3("$x$",0,3,Arrow3);
yaxis3("$y$",0,3,Arrow3);
zaxis3("$z$",0,3,Arrow3);
