defaultpen(fontsize(10 pt));
import smoothcontour3;
import graph3; import grid3; import three; import tube;
import animation; import labelpath3; import contour;
import math;

import texsettings;

size(0,7cm);
currentprojection=perspective((3,-4,4));
currentlight=light(specularfactor=2,(0,0,3),(0,-2,1));

real r1=-1;
real r2=0.25;
real r3=0.5;

real f(real x, real y){
    return y^2-(x-r1)*(x-r2)*(x-r3);
}
guide[][] lp=contour(f,(r1,-1),(r2,1), new real[] {0});
guide[][] li=contour(f,(r3,-1),(1,1), new real[] {0});
guide3 plp;
guide3 alp;
int a=size(lp[0][0]);
for(int k=0;k<a+1;++k){
    pair p = point(lp[0][0],k);
    real phi = acos(1/sqrt(p.x^2+p.y^2+1));
    real theta= angle(p);
    triple q = (cos(theta)*sin(phi),sin(theta)*sin(phi),cos(phi));
    plp=plp..q;
    alp=alp..(p.x,p.y,1);
}
guide3 ali;
int b=size(li[0][0]);
for(int k=0;k<b+1;++k){
    pair p = point(li[0][0],k);
    ali=ali..(p.x,p.y,1);
}

guide3 pli;
int M=1000;
for(int i=1;i<M;++i){
    real theta=-pi/2+pi/M*i;
    pair[] roots=quarticroots(0,
        cos(theta)^2,
        cos(theta)^2*(-r1-r2-r3)+cos(theta)^2-1,
        cos(theta)^2*(r1*r2+r1*r3+r2*r3),
        cos(theta)^2*(-r1*r2*r3));
    real[] roots_real;
    for(int k=0;k<roots.length;++k){
        roots_real[k] = roots[k].x;
    }
    real xa=max(roots_real);
    real ya=sgn(sin(theta))*sqrt((xa-r1)*(xa-r2)*(xa-r3));
    real phi=atan(sqrt(xa^2+ya^2));
    triple p=(cos(theta)*sin(phi),
          sin(theta)*sin(phi),
          cos(phi));
    pli=pli..p;
}

triple c(pair lr){
    real l=lr.x; real r=lr.y;
    return r*arcpoint(plp,l);
}
triple ec(pair lr){
    real l=lr.x; real r=lr.y;
    return (1-r)*unit(arcpoint(alp,l))+r*arcpoint(alp,l);
}
triple s(pair lr){
    real l=lr.x; real r=lr.y;
    return r*arcpoint(pli,l);
}
triple es(pair lr){
    real l=lr.x; real r=lr.y;
    return (1-r)*unit(arcpoint(ali,l))+r*arcpoint(ali,l);
}
path3 linf=arc(
    O,
    X,
   -X,
    Z);

real Lec=arclength(alp);
real Lc=arclength(plp);
real Les=arclength(ali);
real Ls=arclength(pli);

surface cone=surface(c,(0,0),(Lc,1),nu=20,nv=20,Spline);
surface econe=surface(ec,(0,0),(Lec,1),nu=15,nv=15,Spline);
surface sheet=surface(s,(0,0),(Ls,1),nu=20,nv=20,Spline);
surface esheet=surface(es,(0,0),(Les,1),nu=15,nv=15,Spline);
surface plinf=surface(linf..O..cycle);

draw(cone,surfacepen=roseorange*0.6);
draw(econe,surfacepen=roseorange+opacity(0.25));
draw(sheet,surfacepen=roseorange*0.6);
draw(esheet,surfacepen=roseorange+opacity(0.25));

draw(tube(plp,scale(0.00725)*unitcircle),roseorange);
draw(tube(pli,scale(0.00725)*unitcircle),roseorange);

draw(unithemisphere,rosewhite+opacity(0.25));
draw(tube(linf,scale(0.00725)*unitsquare),rosewhite);
//draw(plinf,rosewhite+opacity(0.25));

draw(O--X,arrow=Arrow3,L=Label("$X$", position=EndPoint));
draw(O--Y,roseorange,arrow=Arrow3,L=Label("$Y$", position=EndPoint));
draw(O--Z,arrow=Arrow3,L=Label("$Z$", position=EndPoint));

string txt="the line at infinity $\mathbf{L}$";
string txt2="the projective curve $\mathcal{E}$";
string txt3="the Affine plane $\mathbf{A}$";
string txt4="the affine curve $E$";
draw(labelpath(
    txt,
    subpath(linf,
        reltime(linf,0.4),
        reltime(linf,0.05)),
    angle=-90),
    black);
draw(labelpath(
    txt2,
    subpath(pli,
        reltime(pli,0.05),
        reltime(pli,0.4)),
        angle=0),
    white);
draw(labelpath(
    txt4,
    subpath(ali,
        reltime(ali,0.5),
        reltime(ali,0.95)),
        angle=-90),
    black);
    
//draw(labelpath(
//    txt3,
//    subpath(arc(O,-X,X,2Z+Y),
//        reltime(arc(O,-X,X,2Z+Y),0.2),
//        reltime(arc(O,-X,X,2Z+Y),0.5)),
//        angle=0),
//    black);

draw((-1,-1,1)--(-1,1,1)--(1,1,1)--(1,-1,1)--cycle,black+opacity(0.5));
//draw(surface((-2,-2,1)--(-2,2,1)--(2,2,1)--(2,-2,1)--cycle),rosewhite+opacity(0.25));
draw(alp,roseorange+1+opacity(0.5));
draw(ali,roseorange+1+opacity(0.5));

draw((0,-1,1)--(0,1,1),black+opacity(0.5));
draw((-1,0,1)--(1,0,1),black+opacity(0.5));
